#include "planck.h"
#include "action_layer.h"
#include "eeconfig.h"

#ifdef AUDIO_ENABLE
#include "audio.h"
#endif

extern keymap_config_t keymap_config;

#define _QWERTY 0
#define _COLEMAK 1
#define _DVORAK 2
#define _LOWER 3
#define _RAISE 4
#define _PLOVER 5
#define _ADJUST 16

#define ESC_CTL LCTL_T(KC_ESC)

enum planck_keycodes {
	QWERTY = SAFE_RANGE,
	COLEMAK,
	DVORAK,
	PLOVER,
	LOWER,
	RAISE,
	BACKLIT,
	EXT_PLV
};

#define _______ KC_TRNS
#define XXXXXXX KC_NO

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
	[_QWERTY] = {
		{KC_TAB  , KC_Q    , KC_W    , KC_E    , KC_R   , KC_T  , KC_Y  , KC_U   , KC_I    , KC_O    , KC_P    , KC_BSPC} ,
		{ESC_CTL , KC_A    , KC_S    , KC_D    , KC_F   , KC_G  , KC_H  , KC_J   , KC_K    , KC_L    , KC_SCLN , KC_QUOT} ,
		{KC_LSFT , KC_Z    , KC_X    , KC_C    , KC_V   , KC_B  , KC_N  , KC_M   , KC_COMM , KC_DOT  , KC_SLSH , KC_ENT } ,
		{_______ , KC_LCTL , KC_LALT , KC_LGUI , KC_SPC , LOWER , RAISE , KC_SPC , KC_LEFT , KC_DOWN , KC_UP   , KC_RGHT}
	},
	[_COLEMAK] = {
		{KC_TAB  , KC_Q    , KC_W    , KC_F    , KC_P   , KC_G  , KC_J  , KC_L   , KC_U    , KC_Y    , KC_SCLN , KC_BSPC} ,
		{ESC_CTL , KC_A    , KC_R    , KC_S    , KC_T   , KC_D  , KC_H  , KC_N   , KC_E    , KC_I    , KC_O    , KC_QUOT} ,
		{KC_LSFT , KC_Z    , KC_X    , KC_C    , KC_V   , KC_B  , KC_K  , KC_M   , KC_COMM , KC_DOT  , KC_SLSH , KC_ENT } ,
		{XXXXXXX , KC_LCTL , KC_LALT , KC_LGUI , KC_SPC , LOWER , RAISE , KC_SPC , KC_LEFT , KC_DOWN , KC_UP   , KC_RGHT}
	},
	[_DVORAK] = {
		{KC_TAB  , KC_QUOT , KC_COMM , KC_DOT  , KC_P   , KC_Y  , KC_F  , KC_G   , KC_C    , KC_R    , KC_L  , KC_BSPC} ,
		{ESC_CTL , KC_A    , KC_O    , KC_E    , KC_U   , KC_I  , KC_D  , KC_H   , KC_T    , KC_N    , KC_S  , KC_SLSH} ,
		{KC_LSFT , KC_SCLN , KC_Q    , KC_J    , KC_K   , KC_X  , KC_B  , KC_M   , KC_W    , KC_V    , KC_Z  , KC_ENT } ,
		{XXXXXXX , KC_LCTL , KC_LALT , KC_LGUI , KC_SPC , LOWER , RAISE , KC_SPC , KC_LEFT , KC_DOWN , KC_UP , KC_RGHT}
	},
	[_LOWER] = {
		{KC_TILD , KC_EXLM , KC_AT   , KC_HASH , KC_DLR  , KC_PERC , KC_CIRC , KC_AMPR    , KC_ASTR    , KC_LPRN , KC_RPRN , KC_BSPC} ,
		{KC_DEL  , KC_F1   , KC_F2   , KC_F3   , KC_F4   , KC_F5   , KC_F6   , KC_UNDS    , KC_PLUS    , KC_LCBR , KC_RCBR , KC_PIPE} ,
		{_______ , KC_F7   , KC_F8   , KC_F9   , KC_F10  , KC_F11  , KC_F12  , S(KC_NUHS) , S(KC_NUBS) , _______ , _______ , _______} ,
		{_______ , _______ , _______ , _______ , _______ , _______ , _______ , _______    , KC_HOME    , KC_PGDN , KC_PGUP , KC_END }
	},
	[_RAISE] = {
		{KC_GRV  , KC_1    , KC_2    , KC_3    , KC_4    , KC_5    , KC_6    , KC_7    , KC_8    , KC_9    , KC_0    , KC_DEL } ,
		{KC_DEL  , KC_F1   , KC_F2   , KC_F3   , KC_F4   , KC_F5   , KC_F6   , KC_MINS , KC_EQL  , KC_LBRC , KC_RBRC , KC_BSLS} ,
		{_______ , KC_F7   , KC_F8   , KC_F9   , KC_F10  , KC_F11  , KC_F12  , KC_INS  , KC_NUBS , _______ , _______ , _______} ,
		{_______ , _______ , _______ , _______ , _______ , _______ , _______ , _______ , KC_MNXT , KC_VOLD , KC_VOLU , KC_MPLY}
	},

	/* Plover layer (http://opensteno.org)
	 * ,-----------------------------------------------------------------------------------.
	 * |   #  |   #  |   #  |   #  |   #  |   #  |   #  |   #  |   #  |   #  |   #  |   #  |
	 * |------+------+------+------+------+-------------+------+------+------+------+------|
	 * |      |   S  |   T  |   P  |   H  |   *  |   *  |   F  |   P  |   L  |   T  |   D  |
	 * |------+------+------+------+------+------|------+------+------+------+------+------|
	 * |TogOut|   S  |   K  |   W  |   R  |   *  |   *  |   R  |   B  |   G  |   S  |   Z  |
	 * |------+------+------+------+------+------+------+------+------+------+------+------|
	 * | Exit |      |      |   A  |   O  |             |   E  |   U  |      |      |      |
	 * `-----------------------------------------------------------------------------------'
	 */
	[_PLOVER] = {
		{KC_1    , KC_1    , KC_1    , KC_1 , KC_1 , KC_1    , KC_1    , KC_1 , KC_1 , KC_1    , KC_1    , KC_1   } ,
		{XXXXXXX , KC_Q    , KC_W    , KC_E , KC_R , KC_T    , KC_Y    , KC_U , KC_I , KC_O    , KC_P    , KC_LBRC} ,
		{XXXXXXX , KC_A    , KC_S    , KC_D , KC_F , KC_G    , KC_H    , KC_J , KC_K , KC_L    , KC_SCLN , KC_QUOT} ,
		{EXT_PLV , XXXXXXX , XXXXXXX , KC_C , KC_V , XXXXXXX , XXXXXXX , KC_N , KC_M , XXXXXXX , XXXXXXX , XXXXXXX}
	},

	[_ADJUST] = {
		{RESET   , AG_NORM , AG_SWAP , _______ , _______ , _______ , KC_MNXT , KC_VOLD , KC_VOLU , KC_MPLY , _______ , KC_DEL } ,
		{_______ , _______ , _______ , _______ , _______ , _______ , KC_LEFT , KC_DOWN , KC_UP   , KC_RGHT , _______ , _______} ,
		{MU_ON   , MU_OFF  , MI_ON   , MI_OFF  , _______ , _______ , KC_HOME , KC_PGDN , KC_PGUP , KC_END  , _______ , _______} ,
		{MUV_DE  , MUV_IN  , AU_ON   , AU_OFF  , _______ , _______ , _______ , _______ , QWERTY  , COLEMAK , DVORAK  , PLOVER }
	}
};

#ifdef AUDIO_ENABLE

float tone_startup[][2]    = SONG(STARTUP_SOUND);
float tone_qwerty[][2]     = SONG(QWERTY_SOUND);
float tone_dvorak[][2]     = SONG(DVORAK_SOUND);
float tone_colemak[][2]    = SONG(COLEMAK_SOUND);
float tone_plover[][2]     = SONG(PLOVER_SOUND);
float tone_plover_gb[][2]  = SONG(PLOVER_GOODBYE_SOUND);
float music_scale[][2]     = SONG(MUSIC_SCALE_SOUND);

float tone_goodbye[][2] = SONG(GOODBYE_SOUND);
#endif


void persistant_default_layer_set(uint16_t default_layer) {
	eeconfig_update_default_layer(default_layer);
	default_layer_set(default_layer);
}

bool process_record_user(uint16_t keycode, keyrecord_t *record) {
	switch (keycode) {
		case QWERTY:
			if (record->event.pressed) {
#ifdef AUDIO_ENABLE
				PLAY_NOTE_ARRAY(tone_qwerty, false, 0);
#endif
				persistant_default_layer_set(1UL<<_QWERTY);
			}
			break;
			return false;
		case COLEMAK:
			if (record->event.pressed) {
#ifdef AUDIO_ENABLE
				PLAY_NOTE_ARRAY(tone_colemak, false, 0);
#endif
				persistant_default_layer_set(1UL<<_COLEMAK);
			}
			break;
			return false;
		case DVORAK:
			if (record->event.pressed) {
#ifdef AUDIO_ENABLE
				PLAY_NOTE_ARRAY(tone_dvorak, false, 0);
#endif
				persistant_default_layer_set(1UL<<_DVORAK);
			}
			break;
			return false;
		case LOWER:
			if (record->event.pressed) {
				layer_on(_LOWER);
				update_tri_layer(_LOWER, _RAISE, _ADJUST);
			} else {
				layer_off(_LOWER);
				update_tri_layer(_LOWER, _RAISE, _ADJUST);
			}
			break;
			return false;
		case RAISE:
			if (record->event.pressed) {
				layer_on(_RAISE);
				update_tri_layer(_LOWER, _RAISE, _ADJUST);
			} else {
				layer_off(_RAISE);
				update_tri_layer(_LOWER, _RAISE, _ADJUST);
			}
			break;
			return false;
		case BACKLIT:
			if (record->event.pressed) {
				register_code(KC_RSFT);
#ifdef BACKLIGHT_ENABLE
				backlight_step();
#endif
			} else {
				unregister_code(KC_RSFT);
			}
			break;
			return false;
		case PLOVER:
			if (record->event.pressed) {
#ifdef AUDIO_ENABLE
				stop_all_notes();
				PLAY_NOTE_ARRAY(tone_plover, false, 0);
#endif
				layer_off(_RAISE);
				layer_off(_LOWER);
				layer_off(_ADJUST);
				layer_on(_PLOVER);
				if (!eeconfig_is_enabled()) {
					eeconfig_init();
				}
				keymap_config.raw = eeconfig_read_keymap();
				keymap_config.nkro = 1;
				eeconfig_update_keymap(keymap_config.raw);
			}
			break;
			return false;
		case EXT_PLV:
			if (record->event.pressed) {
#ifdef AUDIO_ENABLE
				PLAY_NOTE_ARRAY(tone_plover_gb, false, 0);
#endif
				layer_off(_PLOVER);
			}
			break;
			return false;
	}
	return true;
}

void matrix_init_user(void) {
#ifdef AUDIO_ENABLE
	startup_user();
#endif
}

#ifdef AUDIO_ENABLE
void startup_user()
{
	_delay_ms(20); // gets rid of tick
	PLAY_NOTE_ARRAY(tone_startup, false, 0);
}

void shutdown_user()
{
	PLAY_NOTE_ARRAY(tone_goodbye, false, 0);
	_delay_ms(150);
	stop_all_notes();
}

void music_on_user(void)
{
	music_scale_user();
}

void music_scale_user(void)
{
	PLAY_NOTE_ARRAY(music_scale, false, 0);
}
#endif
